.progress-container .bx--progress-bar__label,.progress-container .bx--progress-bar__helper-text{font-size:.75rem;line-height:1rem;font-weight:300}.task-button .bx--popover-contents{width:28rem;max-width:28rem}
