.embedding-badge-nolabel .bx--tooltip__label .bx--tooltip__trigger{margin:0}.embedding-badge-nolabel .bx--tag__custom-icon{margin-right:0}.named-value-name.svelte-1689hje{max-width:15rem}.highlight-span.svelte-8ox5pu{padding-top:1.5px;padding-bottom:1.5px}.highlight-span pre{--tw-bg-opacity:1;background-color:rgb(226 232 240 / var(--tw-bg-opacity));font-size:.875rem;line-height:1.25rem}.highlight-span p,.highlight-span pre{margin-top:.75rem;margin-bottom:.75rem}.highlight-span p:first-child{display:inline!important}.highlight-span p:last-child{display:inline!important}.highlight-span p,.highlight-span h1{background-color:inherit}.highlight-span p{font-size:.875rem;line-height:1.25rem;font-weight:inherit}
