import{ao as q,S as M,i as E,s as F,E as h,Z as g,_,m,h as u,n as A,$ as p,b as C,a0 as N,L as T,a1 as v,N as b,Q as w,q as O,r as S,u as B,P as U,am as W,ai as X}from"./index.8706e7a3.js";import{w as D}from"./index.8b48d14e.js";function Q(s){const e=s-1;return e*e*e+1}function V(s,{delay:e=0,duration:i=400,easing:t=q}={}){const n=+getComputedStyle(s).opacity;return{delay:e,duration:i,easing:t,css:o=>`opacity: ${o*n}`}}function $(s,{delay:e=0,duration:i=400,easing:t=Q,axis:n="y"}={}){const o=getComputedStyle(s),a=+o.opacity,r=n==="y"?"height":"width",f=parseFloat(o[r]),l=n==="y"?["top","bottom"]:["left","right"],c=l.map(d=>`${d[0].toUpperCase()}${d.slice(1)}`),I=parseFloat(o[`padding${c[0]}`]),R=parseFloat(o[`padding${c[1]}`]),k=parseFloat(o[`margin${c[0]}`]),Y=parseFloat(o[`margin${c[1]}`]),L=parseFloat(o[`border${c[0]}Width`]),P=parseFloat(o[`border${c[1]}Width`]);return{delay:e,duration:i,easing:t,css:d=>`overflow: hidden;opacity: ${Math.min(d*20,1)*a};${r}: ${d*f}px;padding-${l[0]}: ${d*I}px;padding-${l[1]}: ${d*R}px;margin-${l[0]}: ${d*k}px;margin-${l[1]}: ${d*Y}px;border-${l[0]}-width: ${d*L}px;border-${l[1]}-width: ${d*P}px;`}}function z(s){let e,i;return{c(){e=g("title"),i=O(s[1])},l(t){e=_(t,"title",{});var n=m(e);i=S(n,s[1]),n.forEach(u)},m(t,n){C(t,e,n),N(e,i)},p(t,n){n&2&&B(i,t[1])},d(t){t&&u(e)}}}function Z(s){let e,i,t=s[1]&&z(s),n=[{xmlns:"http://www.w3.org/2000/svg"},{viewBox:"0 0 32 32"},{fill:"currentColor"},{preserveAspectRatio:"xMidYMid meet"},{width:s[0]},{height:s[0]},s[2],s[3]],o={};for(let a=0;a<n.length;a+=1)o=h(o,n[a]);return{c(){e=g("svg"),t&&t.c(),i=g("path"),this.h()},l(a){e=_(a,"svg",{xmlns:!0,viewBox:!0,fill:!0,preserveAspectRatio:!0,width:!0,height:!0});var r=m(e);t&&t.l(r),i=_(r,"path",{d:!0}),m(i).forEach(u),r.forEach(u),this.h()},h(){A(i,"d","M16 22L6 12 7.4 10.6 16 19.2 24.6 10.6 26 12z"),p(e,o)},m(a,r){C(a,e,r),t&&t.m(e,null),N(e,i)},p(a,[r]){a[1]?t?t.p(a,r):(t=z(a),t.c(),t.m(e,i)):t&&(t.d(1),t=null),p(e,o=T(n,[{xmlns:"http://www.w3.org/2000/svg"},{viewBox:"0 0 32 32"},{fill:"currentColor"},{preserveAspectRatio:"xMidYMid meet"},r&1&&{width:a[0]},r&1&&{height:a[0]},r&4&&a[2],r&8&&a[3]]))},i:v,o:v,d(a){a&&u(e),t&&t.d()}}}function j(s,e,i){let t,n;const o=["size","title"];let a=b(e,o),{size:r=16}=e,{title:f=void 0}=e;return s.$$set=l=>{i(5,e=h(h({},e),w(l))),i(3,a=b(e,o)),"size"in l&&i(0,r=l.size),"title"in l&&i(1,f=l.title)},s.$$.update=()=>{i(4,t=e["aria-label"]||e["aria-labelledby"]||f),i(2,n={"aria-hidden":t?void 0:!0,role:t?"img":void 0,focusable:Number(e.tabindex)===0?!0:void 0})},e=w(e),[r,f,n,a,t]}class ee extends M{constructor(e){super(),E(this,e,j,Z,F,{size:0,title:1})}}function x(s){let e,i;return{c(){e=g("title"),i=O(s[1])},l(t){e=_(t,"title",{});var n=m(e);i=S(n,s[1]),n.forEach(u)},m(t,n){C(t,e,n),N(e,i)},p(t,n){n&2&&B(i,t[1])},d(t){t&&u(e)}}}function G(s){let e,i,t=s[1]&&x(s),n=[{xmlns:"http://www.w3.org/2000/svg"},{viewBox:"0 0 32 32"},{fill:"currentColor"},{preserveAspectRatio:"xMidYMid meet"},{width:s[0]},{height:s[0]},s[2],s[3]],o={};for(let a=0;a<n.length;a+=1)o=h(o,n[a]);return{c(){e=g("svg"),t&&t.c(),i=g("path"),this.h()},l(a){e=_(a,"svg",{xmlns:!0,viewBox:!0,fill:!0,preserveAspectRatio:!0,width:!0,height:!0});var r=m(e);t&&t.l(r),i=_(r,"path",{d:!0}),m(i).forEach(u),r.forEach(u),this.h()},h(){A(i,"d","M16 10L26 20 24.6 21.4 16 12.8 7.4 21.4 6 20z"),p(e,o)},m(a,r){C(a,e,r),t&&t.m(e,null),N(e,i)},p(a,[r]){a[1]?t?t.p(a,r):(t=x(a),t.c(),t.m(e,i)):t&&(t.d(1),t=null),p(e,o=T(n,[{xmlns:"http://www.w3.org/2000/svg"},{viewBox:"0 0 32 32"},{fill:"currentColor"},{preserveAspectRatio:"xMidYMid meet"},r&1&&{width:a[0]},r&1&&{height:a[0]},r&4&&a[2],r&8&&a[3]]))},i:v,o:v,d(a){a&&u(e),t&&t.d()}}}function H(s,e,i){let t,n;const o=["size","title"];let a=b(e,o),{size:r=16}=e,{title:f=void 0}=e;return s.$$set=l=>{i(5,e=h(h({},e),w(l))),i(3,a=b(e,o)),"size"in l&&i(0,r=l.size),"title"in l&&i(1,f=l.title)},s.$$.update=()=>{i(4,t=e["aria-label"]||e["aria-labelledby"]||f),i(2,n={"aria-hidden":t?void 0:!0,role:t?"img":void 0,focusable:Number(e.tabindex)===0?!0:void 0})},e=w(e),[r,f,n,a,t]}class te extends M{constructor(e){super(),E(this,e,H,G,F,{size:0,title:1})}}const y="NOTIFICATIONS_CONTEXT";function ie(){const{subscribe:s,set:e,update:i}=D({notifications:[]});return{subscribe:s,set:e,update:i,reset(){e({notifications:[]})},addNotification(t){i(n=>(n.notifications.push(t),n))},removeNotification(t){i(n=>(n.notifications=n.notifications.filter(o=>o!==t),n))}}}function ae(s){U(y,s)}function ne(){if(!W(y))throw new Error("NotificationContext not found");return X(y)}export{ee as C,te as a,ae as b,ie as c,V as f,ne as g,$ as s};
