.description p{font-size:.875rem;line-height:1.25rem;margin:1em 0}.bx--toggle-input__label .bx--toggle__switch{margin-top:0}button.svelte-d3v0kx{width:100%;padding:.5rem 1rem;text-align:left;--tw-text-opacity:1;color:rgb(31 41 55 / var(--tw-text-opacity))}button.svelte-d3v0kx:hover{--tw-bg-opacity:1;background-color:rgb(229 231 235 / var(--tw-bg-opacity));--tw-text-opacity:1;color:rgb(0 0 0 / var(--tw-text-opacity))}button[data-active=true].svelte-d3v0kx{--tw-bg-opacity:1;background-color:rgb(209 213 219 / var(--tw-bg-opacity));--tw-text-opacity:1;color:rgb(0 0 0 / var(--tw-text-opacity))}.bx--tag{margin:0}.text-container.svelte-1a6oxtp{min-width:4rem;max-width:16rem}.header-step.svelte-62ihoh{margin-bottom:.5rem;font-size:1rem;line-height:1.5rem}.generate-button .bx--btn{height:2.5rem;min-height:0px}
