import{S as x,i as S,s as g,k as u,q as h,a as k,l as d,m as v,r as b,h as m,c as y,b as _,a0 as E,u as $,a1 as q,O as C}from"../chunks/index.8706e7a3.js";import{p as H}from"../chunks/stores.c19e0230.js";function O(l){var f;let a,t=l[0].status+"",r,o,n,p=((f=l[0].error)==null?void 0:f.message)+"",c;return{c(){a=u("h1"),r=h(t),o=k(),n=u("p"),c=h(p)},l(e){a=d(e,"H1",{});var s=v(a);r=b(s,t),s.forEach(m),o=y(e),n=d(e,"P",{});var i=v(n);c=b(i,p),i.forEach(m)},m(e,s){_(e,a,s),E(a,r),_(e,o,s),_(e,n,s),E(n,c)},p(e,[s]){var i;s&1&&t!==(t=e[0].status+"")&&$(r,t),s&1&&p!==(p=((i=e[0].error)==null?void 0:i.message)+"")&&$(c,p)},i:q,o:q,d(e){e&&m(a),e&&m(o),e&&m(n)}}}function P(l,a,t){let r;return C(l,H,o=>t(0,r=o)),[r]}class z extends x{constructor(a){super(),S(this,a,P,O,g,{})}}export{z as component};
